#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file WiFiRouteObserver.py
 @brief This RTC observes WiFi IP routing table information.
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import sys
import copy
import uuid
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

# Import RTM module
import RTC
import OpenRTM_aist
import IPRouteIF

import ip_route_list

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(LOG_LV)
logger.setLevel(LOG_LV)
logger.addHandler(handler)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
wifirouteobserver_spec = ["implementation_id", "WiFiRouteObserver",
                          "type_name",         "WiFiRouteObserver",
                          "description",       "This RTC observes WiFi IP \
                                                routing table information.",
                          "version",           "1.0",
                          "vendor",            "EAC",
                          "category",          "Network",
                          "activity_type",     "STATIC",
                          "max_instance",      "1",
                          "language",          "Python",
                          "lang_type",         "SCRIPT",
                          "conf.default.node_id", "node_id_xxx",
                          "conf.__widget__.node_id", "text",
                          ""]
# </rtc-template>

##
# @class WiFiRouteObserver
# @brief This RTC observes WiFi IP routing table information.
#
# 別紙"機能仕様書_Wi-Fi経路情報監視"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報監視"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報監視"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報監視"参照
#
#


class WiFiRouteObserver(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_IPRouteListOut = IPRouteIF.TimedRouteList(
            RTC.Time(0, 0), 0, [])
        """
        IP経路テーブル情報を出力する
         - Type: IPRouteIF::TimedRouteList
         - Number: 動的確保
         - Frequency: コンポーネント実行周期と同様
         - Operation Cycle: コンポーネント実行周期と同様
        """
        self._IPRouteListOutOut = OpenRTM_aist.OutPort(
            "IPRouteListOut", self._d_IPRouteListOut)

        self._ip_route_list = ip_route_list.IPRouteList()

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        コンポーネントノードID。
        IPRouteList.idに使用される。
         - Name: node_id node_id
         - DefaultValue: node_id_xxx
         - Constraint: configuration fileに以下のように事前に設定する事
                       設定されていない場合、MACアドレスが代わりに使用される。
                       conf.default.node_id: drone_01
        """
        self._node_id = ['node_id_xxx']

        # </rtc-template>

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("node_id", self._node_id, "node_id_xxx")

        # Set InPort buffers

        # Set OutPort buffers
        self.addOutPort("IPRouteListOut", self._IPRouteListOutOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports
        # Setup environment for each platforms
        is_success = self._ip_route_list.configure()

        if not is_success:
            return RTC.RTC_ERROR

        # Set node id
        self._set_node_id()
        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The activated action (Active state entry action)
    #    # former rtc_active_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onActivated(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The deactivated action (Active state exit action)
    #    # former rtc_active_exit()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # IP経路テーブル情報を取得する周期処理
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre プラットフォームがWindowsまたはLinux
        # @post 事前条件に合致しない場合にはエラー状態へ遷移
        #
    def onExecute(self, ec_id):
        is_success = self._renew_ip_route_list()

        if not is_success:
            return RTC.RTC_ERROR
        return RTC.RTC_OK

        ##
        # エラーログ出力
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onAborting(self, ec_id):
        logger.error("ERROR: detected an Aborting state")
        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _renew_ip_route_list(self):
        result = self._ip_route_list.get_route_list()

        if not result.is_success:
            return False

        OpenRTM_aist.setTimestamp(self._d_IPRouteListOut)

        self._set_node_id()

        if self._d_IPRouteListOut.data != result.data:
            del self._d_IPRouteListOut.data[:]

            self._d_IPRouteListOut.data = copy.deepcopy(result.data)

            logger.info("updated IP route list:")
            logger.info(self._d_IPRouteListOut.tm)
            for data in self._d_IPRouteListOut.data:
                logger.info(data)

        self._IPRouteListOutOut.write()

        return True

    def _set_node_id(self):
        set_node_id = self._node_id[0]
        if self._node_id[0] == 'node_id_xxx':
            set_node_id = str(uuid.getnode())

        if self._d_IPRouteListOut.id != set_node_id:
            self._d_IPRouteListOut.id = set_node_id

            if set_node_id == str(uuid.getnode()):
                logger.warning("WARNING: node id "
                               "should be set in configuration file")
                logger.warning("WARNING: MAC address is used instead")

            logger.info("new node id: " + self._d_IPRouteListOut.id)


def WiFiRouteObserverInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=wifirouteobserver_spec)
    manager.registerFactory(profile,
                            WiFiRouteObserver,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    WiFiRouteObserverInit(manager)

    # Create a component
    manager.createComponent("WiFiRouteObserver")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
