#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file WiFiRouteViewer.py
 @brief This RTC displays IP routing table information and netowork topology.
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import sys
import copy
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

# Import RTM module
import RTC
import OpenRTM_aist

import IPRouteIF
from tk_ip_route_viewer import TkIPRouteViewer, NODE, ROUTE, INFO, ALL
from wifi_nw_image_generator import GenerateTopologyImage

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Setup tk_viewer
tk_viewer = TkIPRouteViewer()
img_gen = GenerateTopologyImage()

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
wifirouteviewer_spec = ["implementation_id", "WiFiRouteViewer",
                        "type_name",         "WiFiRouteViewer",
                        "description",       "This RTC displays IP routing \
                                              table information and netowork \
                                              topology.",
                        "version",           "1.0.0",
                        "vendor",            "EAC",
                        "category",          "Network",
                        "activity_type",     "STATIC",
                        "max_instance",      "1",
                        "language",          "Python",
                        "lang_type",         "SCRIPT",
                        "conf.default.base_node_id", "base_node_id_xx",
                        "conf.__widget__.base_node_id", "text",
                        ""]
# </rtc-template>

##
# @class WiFiRouteViewer
# @brief This RTC displays IP routing table information and netowork topology.
#
# 別紙"機能仕様書_Wi-Fi経路情報ビューア"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報ビューア"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報ビューア"参照
#
# 別紙"機能仕様書_Wi-Fi経路情報ビューア"参照
#
#


class WiFiRouteViewer(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_IPRouteListIn = IPRouteIF.TimedRouteList(RTC.Time(0, 0), 0, [])
        """
        IP経路テーブル情報を入力する
         - Type: IPRouteIF::TimedRouteList
         - Semantics: -
         - Unit: -
         - Frequency: コンポーネント実行周期と同様
         - Operation Cycle: コンポーネント実行周期と同様
        """
        self._IPRouteListInIn = OpenRTM_aist.InPort(
            "IPRouteListIn", self._d_IPRouteListIn)

        self._init_variable()
        tk_viewer.init_viewer()

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        基地局nodeの名称
         - Name:  base_node_id
         - DefaultValue: base_node_id_xx
        """
        self._base_node_id = ['base_node_id_xx']

        # </rtc-template>

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre -
    # @post -
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter(
            "base_node_id", self._base_node_id, "base_node_id_xx")

        # Set InPort buffers
        self.addInPort("IPRouteListIn", self._IPRouteListInIn)

        # Set OutPort buffers

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # データリスナー登録
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onActivated(self, ec_id):
        self._listener = DataListener(self._cb_rcv_route_data)
        listener_type = OpenRTM_aist.ConnectorDataListenerType.ON_RECEIVED

        self._IPRouteListInIn.addConnectorDataListener(
            listener_type, self._listener)
        return RTC.RTC_OK

        ##
        # データリスナー登録解除
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onDeactivated(self, ec_id):
        listener_type = OpenRTM_aist.ConnectorDataListenerType.ON_RECEIVED

        self._IPRouteListInIn.removeConnectorDataListener(
            listener_type, self._listener)

        self._init_variable()
        tk_viewer.init_viewer()

        return RTC.RTC_OK

        ##
        # 孤立ノード検知周期処理、トポロジー図生成表示処理
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onExecute(self, ec_id):
        # print "<<onExecute()>>"
        self._check_missing_node()
        self._check_base_node()

        if self._img_create:
            self._img_create = False
            img_gen.generate_image()
            tk_viewer.draw_nwtopology()

        return RTC.RTC_OK

        ##
        # エラーログ出力
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onAborting(self, ec_id):
        logger.error("ERROR: detected an Aborting state")
        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _init_variable(self):
        self._route_list = []
        self._route_list_bk = []
        self._listener = None
        self._missing_node_list = []  # h28-n
        # self._missing_node_list_bk = []     #h28-n
        self._img_create = False  # h28-n
        self._node_pos = False
        self._base_node_id_bk = ['base_node_id_xx']

    def get_route_list(self):
        # h28-n
        return self._route_list

    def get_missing_node_list(self):
        # h28-n
        return self._missing_node_list

    def get_sel_node(self):
        # h28-n
        return tk_viewer.get_node_id()

    def get_base_node_id(self):
        # h28-n
        # print "<<get_base_node_id>>"
        temp = copy.deepcopy(self._base_node_id)
        # temp = ip_a[0].replace(" ", "")
        ret = temp
        return ret

    def get_node_pos_change(self):
        retval = False
        if self._node_pos:
            self._node_pos = False
            retval = True
        return retval

    def get_route_data(self, node_id):
        ret = None
        for data in self._route_list:
            if node_id == data.id:
                ret = data
                break

        if not ret:
            logger.error("invalid requested node_id: " + node_id)

        return ret

    def set_sel_node_change(self):
        self._img_create = True

    def _check_missing_node(self):
        # missing node list clear
        # del self._missing_node_list[:]  # h28-n

        for i, (r1, r2) in enumerate(zip(self._route_list,
                                         self._route_list_bk)):
            if r1 == r2:
                # no change data
                # print "no change route data:{0}".format(r1.id)
                """
                if r1.data:
                    del r1.data[:]
                    tk_viewer.draw_gui(ALL, r1.id, missing=True)
                    logger.info("missing node(%d): %s", i, r1.id)
                """
                if r1.id not in self._missing_node_list:
                    del r1.data[:]
                    logger.info("missing node(%d): %s", i, r1.id)
                    # append missing node list
                    # print "append missing node lisst id:{0}".format(r1.id)
                    self._missing_node_list.append(r1.id)  # h28-n
                    self._img_create = True

                    tk_viewer.draw_gui(ALL, r1.id, missing=True)

            else:
                # change data
                # print "change route data:{0}".format(r1.id)
                if r1.id in self._missing_node_list:
                    # remove missing node list
                    # print "remove missing node lisst id:{0}".format(r1.id)
                    self._missing_node_list.remove(r1.id)
                    self._img_create = True

        # print "###_missing_node_list={0}".format(self._missing_node_list)
        self._route_list_bk = copy.deepcopy(self._route_list)

    def _check_base_node(self):
        if self._base_node_id_bk[0] == self._base_node_id[0]:
            pass
        else:
            self._base_node_id_bk[0] = self._base_node_id[0]
            self._img_create = True

    def _cb_rcv_route_data(self, rcv_route_list):
        # print "<<_cb_rcv_route_data()>>"
        is_new = True
        for i, route in enumerate(self._route_list):
            if route.id == rcv_route_list.id:
                # Notice from known Node
                self._route_list[i] = copy.deepcopy(rcv_route_list)
                is_new = False

                if route.data == rcv_route_list.data:
                    # no route change
                    # print "no route change"
                    tk_viewer.draw_gui(INFO, route.id)
                elif route.data:
                    # route change
                    # print "route change"

                    tk_viewer.draw_gui(ROUTE | INFO, route.id)
                    self._img_create = True  # h28-n

                    logger.info("changed the route data of node(%d): %s",
                                i, route.id)
                    logger.info(rcv_route_list)
                else:
                    # found missing node
                    tk_viewer.draw_gui(ALL, route.id, missing=False)
                    logger.info("found the missing node(%d): %s", i, route.id)
                    logger.info(rcv_route_list)
                    self._img_create = True  # h28-n

                break

        if is_new:
            # Notice from unknown Node
            self._route_list.append(rcv_route_list)
            logger.info("new node(%d): %s",
                        len(self._route_list), rcv_route_list.id)
            logger.info(rcv_route_list)

            tk_viewer.draw_gui(NODE, rcv_route_list.id)
            self._img_create = True  # h28-n
            self._node_pos = True


class DataListener(OpenRTM_aist.ConnectorDataListenerT):

    def __init__(self, func):
        self._plugin_func = func
        logger.info("added DataListener")

    def __del__(self):
        logger.info("removed DataListener")

    def __call__(self, info, cdrdata):
        route = IPRouteIF.TimedRouteList(RTC.Time(0, 0), 0, [])
        route_list = OpenRTM_aist.ConnectorDataListenerT.__call__(self, info,
                                                                  cdrdata,
                                                                  route)

        self._plugin_func(route_list)  # WiFiRouteViewer._cb_rcv_route_data()


def WiFiRouteViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=wifirouteviewer_spec)
    manager.registerFactory(profile,
                            WiFiRouteViewer,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    WiFiRouteViewerInit(manager)

    # Create a component
    comp = manager.createComponent("WiFiRouteViewer")
    tk_viewer.set_data_src_func(comp.get_route_data)
    tk_viewer.set_sel_node_change_func(comp.set_sel_node_change)
    tk_viewer.set_missing_node_list_func(comp.get_missing_node_list)

    img_gen.set_sel_node_func(comp.get_sel_node)
    img_gen.set_route_list_func(comp.get_route_list)
    img_gen.set_missing_node_list_func(comp.get_missing_node_list)
    img_gen.set_base_node_id_func(comp.get_base_node_id)
    img_gen.set_pos_change_func(comp.get_node_pos_change)


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    tk_viewer.mainloop()
    mgr.shutdown()


if __name__ == "__main__":
    main()
