#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file wifi_nw_image_generator.py
 @brief This module is generete netowork toplogy image file(png)
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import os.path
import sys
from datetime import datetime
import matplotlib
matplotlib.rcParams['backend.qt4'] = 'PySide'
matplotlib.use('Qt4Agg')

import matplotlib.pyplot as plt
import networkx as nx
import random
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

B_COLOR = "limegreen"   # Base node color
SEL_COLOR = "deepskyblue"   # selected node color
R_COLOR = "orange"      # Robot node color
MISSING_COLOR = "red"   # missing node color

B_SIZE = 2500           # Base node size
R_SIZE = 1500           # Robot node size

FONT_COLOR = "blue"    # node name font color
FONT_SIZE = 18          # node name font size

EDGE_COLOR = "black"
EDGE_WIDTH = 3          # edge width

CURRENT = os.getcwd()
IMG_PATH = CURRENT + "/WiFi_network_topology.png"

G = nx.Graph()


class GenerateTopologyImage():

    def __init__(self):
        print "name of the current matplotlib backend =" +\
            matplotlib.get_backend()

        self._init_variable()

    def _init_variable(self):
        self._node_ip_list = []
        self._base_node_id = ""
        self._edge_list = []
        self._edge_list2 = []
        self._get_sel_node_func = None
        self._get_route_list_func = None
        self._get_missing_node_list_func = None
        self._get_base_node_id_func = None
        self._get_pos_change_func = None
        self._pos_change = False
        self._gpos = ""
        self._sel_id = ""
        self._activtim = datetime.now().isoformat().replace(":", ".")

    def generate_image(self):
        route_list = self._get_route_list_func()
        missing_list = self._get_missing_node_list_func()
        self._sel_id = self._get_sel_node_func()
        self._base_node_id = self._get_base_node_id_func()
        self._pos_change = self._get_pos_change_func()

        self._node_ip_list_create(route_list)
        self._node_edge_list_create(route_list)

        self._topology_image_draw(missing_list)

    def _node_ip_list_create(self, r_list):

        del self._node_ip_list[:]
        for r in r_list:
            data_ip = []
            for v in r.data:
                if v.iface in data_ip:
                    pass
                else:
                    data_ip.append(v.iface)

            appdata = (r.id, data_ip)
            self._node_ip_list.append(appdata)

    def _node_edge_list_create(self, r_list):
        del self._edge_list[:]
        del self._edge_list2[:]
        base_node_notice = None
        base_node_ip = None
        for y1, y2 in self._node_ip_list:
            if self._base_node_id[0] == y1:
                # notification from base node
                base_node_notice = True
                base_node_ip = y2

        if not base_node_notice:
            return

        for r in r_list:
            flg_append = None

            for v in r.data:
                dest = v.destination

                r1 = self._find_node_id(v.gateway)
                r2 = self._find_node_id(v.iface)

                if not r1 or not r2 or not dest:
                    pass
                else:
                    if dest in base_node_ip:
                        temp_edge_list = (r1, r2, "black")
                        self._edge_list.append(temp_edge_list)
                        flg_append = True
                    else:
                        temp_edge_list = (r1, r2, "blue")
                        self._edge_list2.append(temp_edge_list)

            if not flg_append:
                missing_list = self._get_missing_node_list_func()
                if r.id not in missing_list and base_node_notice:
                    if r.id == self._base_node_id[0]:
                        pass
                    else:
                        temp_edge_list = (r.id, self._base_node_id[0], "black")
                        self._edge_list.append(temp_edge_list)


    def _find_node_id(self, val):

        ret = False
        for i, d in self._node_ip_list:
            if val in d:
                ret = i
                break

        return ret

    def _topology_image_draw(self, missing_list):

        n_sizes = []
        n_colors = []

        self._node_add_graph(self._node_ip_list, missing_list)

        if self._edge_list:
            if self._edge_list2:
                for i, u, r in self._edge_list2:
                    G.add_edge(i, u, color=r, weight=1)

            for i, u, r in self._edge_list:
                # G.add_edge(i,u,color="black",weight=4)
                G.add_edge(i, u, color=r, weight=EDGE_WIDTH)

            edges = G.edges()
            e_colors = [G[u][v]['color'] for u, v in edges]
            weights = [G[u][v]['weight'] for u, v in edges]
        else:
            e_colors = "black"
            weights = 1

        self._node_attr_set(n_colors, n_sizes)
        if self._pos_change:
            self._gpos = nx.spring_layout(G)
            self._pos_change = False
        nx.draw(G, pos=self._gpos, with_labels=True,
                font_color=FONT_COLOR, edge_color=e_colors, width=weights,
                node_size=n_sizes, node_color=n_colors)

        plt.savefig(IMG_PATH)                        # Save the current figure.
        plt.clf()
        G.clear()

    def _node_attr_set(self, colors, sizes):
        nodes = G.nodes(data=True)

        for i, attr in nodes:
            colors.append(attr["node_color"])
            sizes.append(attr["node_size"])


    def _node_add_graph(self, list, miss_list):

        for i, v in list:
            siz = R_SIZE
            col = R_COLOR
            if i == self._base_node_id[0]:
                col = B_COLOR
                siz = B_SIZE
            if i == self._sel_id:
                col = SEL_COLOR
            for u in miss_list:
                if u == i:
                    # missing node
                    col = MISSING_COLOR

            G.add_node(i, node_size=siz, node_color=col)

    def set_sel_node_func(self, func):
        self._get_sel_node_func = func

    def set_route_list_func(self, func):
        self._get_route_list_func = func

    def set_missing_node_list_func(self, func):
        self._get_missing_node_list_func = func

    def set_base_node_id_func(self, func):
        self._get_base_node_id_func = func

    def set_pos_change_func(self, func):
        self._get_pos_change_func = func


def main():
    pass


if __name__ == "__main__":
    main()
